from multiprocessing import Pool

def escapes(cr, ci, it):
    """
    Does iterating z <- z^2 + c escape after it iterations?
    
    Pure function.
    """
    zr = 0.0
    zi = 0.0
    for i in range(it):
        # z <- z^2 + c
        zr,zi = zr*zr - zi*zi + cr, 2*zr*zi + ci
        if zr*zr + zi*zi > 4:
            return True
    return False

def toChar(p):
    if p:
        return " "
    else:
        return "X"

def mandel(xmin,xmax,xstep, ymin,ymax,ystep, iterations):
    """
    Calculate and print a Mandelbrot set.
    
    (Does this by sending each individual pixel to a separate worker: too small a chunk.)
    """
    pool = Pool()
    
    for yc in range(ystep):
        y = yc*(ymax-ymin)/ystep + ymin
        row = []
        results = []
        for xc in range(xstep):
            x = xc*(xmax-xmin)/xstep + xmin
            res = pool.apply_async(escapes, (x, y, iterations))
            results.append(res)

        for xc in range(xstep):
            row.append(results[xc].get())

        print("".join([toChar(p) for p in row]))

if __name__=="__main__":
    mandel(-2.0, 1.0, 100, -1.0, 1.0, 60, 20000)
